// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:dotted_border/dotted_border.dart';
import 'package:flutter/material.dart' hide ButtonStyle;
import 'package:flutter/services.dart';
import 'package:gap/gap.dart';
import 'package:wyatt_bloc_helper/wyatt_bloc_helper.dart';
import 'package:wyatt_ui_components/wyatt_wyatt_ui_components.dart';
import 'package:wyatt_ui_kit/src/components/buttons/cubit/button_cubit.dart';
import 'package:wyatt_ui_kit/src/components/buttons/cubit/invalid_button_cubit.dart';
import 'package:wyatt_ui_kit/src/components/gradients/gradient_text.dart';
import 'package:wyatt_ui_kit/src/core/extensions/theme_extensions.dart';
import 'package:wyatt_ui_kit/src/core/helpers/linear_gradient_helper.dart';
import 'package:wyatt_ui_kit/src/domain/button_theme_extension/file_selection_button_theme_extension.dart';

class FileSelectionButtonScreen
    extends CubitScreen<InvalidButtonCubit, ButtonState> {
  const FileSelectionButtonScreen({
    this.leading,
    this.title,
    this.subTitle,
    this.disabledStyle,
    this.normalStyle,
    this.hoveredStyle,
    this.focusedStyle,
    this.tappedStyle,
    this.selectedStyle,
    this.invalidStyle,
    this.onPressed,
    this.mainAxisSize,
    super.key,
  });

  final Widget? leading;
  final TextWrapper? title;
  final TextWrapper? subTitle;
  final MainAxisSize? mainAxisSize;

  final FileSelectionButtonStyle? disabledStyle;
  final FileSelectionButtonStyle? normalStyle;
  final FileSelectionButtonStyle? hoveredStyle;
  final FileSelectionButtonStyle? focusedStyle;
  final FileSelectionButtonStyle? tappedStyle;
  final FileSelectionButtonStyle? selectedStyle;
  final FileSelectionButtonStyle? invalidStyle;

  final void Function(ControlState state)? onPressed;

  @override
  InvalidButtonCubit create(BuildContext context) => InvalidButtonCubit();

  /// Negotiate the theme to get a complete style.
  FileSelectionButtonStyle negotiate(BuildContext context, ButtonState state) {
    // Define default style from Flutter values.
    FileSelectionButtonStyle style =
        FileSelectionButtonStyle.fromFlutter(context);

    // Try to retrieve custom theme extension
    final fileSelectionButtonThemeExtension =
        context.themeExtension<FileSelectionButtonThemeExtension>();

    switch (state.state) {
      case ControlState.disabled:
        style = disabledStyle ??
            fileSelectionButtonThemeExtension?.disabledStyle ??
            style.copyWith(
              foregroundColors:
                  MultiColor.single(context.colorScheme.onSurface),
              backgroundColors: MultiColor.single(context.colorScheme.surface),
            );
        break;
      case ControlState.hovered:
        style = hoveredStyle ??
            fileSelectionButtonThemeExtension?.hoveredStyle ??
            style;
        break;
      case ControlState.tapped:
        style = tappedStyle ??
            fileSelectionButtonThemeExtension?.tappedStyle ??
            style;
        break;
      case ControlState.focused:
        style = focusedStyle ??
            fileSelectionButtonThemeExtension?.focusedStyle ??
            style;
        break;
      case ControlState.normal:
        style = normalStyle ??
            fileSelectionButtonThemeExtension?.normalStyle ??
            style;
        break;
    }

    // Apply extra theme
    if (state.isSelected) {
      // TODO(hpcl): enhance copyWith to copy only non-null attributes of an object
      style = style.copyWith(
        title: (selectedStyle ??
                fileSelectionButtonThemeExtension?.selectedStyle ??
                style)
            .title,
        subTitle: (selectedStyle ??
                fileSelectionButtonThemeExtension?.selectedStyle ??
                style)
            .subTitle,
        radius: (selectedStyle ??
                fileSelectionButtonThemeExtension?.selectedStyle ??
                style)
            .radius,
        padding: (selectedStyle ??
                fileSelectionButtonThemeExtension?.selectedStyle ??
                style)
            .padding,
        foregroundColors: (selectedStyle ??
                fileSelectionButtonThemeExtension?.selectedStyle ??
                style)
            .foregroundColors,
        backgroundColors: (selectedStyle ??
                fileSelectionButtonThemeExtension?.selectedStyle ??
                style)
            .backgroundColors,
        borderColors: (selectedStyle ??
                fileSelectionButtonThemeExtension?.selectedStyle ??
                style)
            .borderColors,
        stroke: (selectedStyle ??
                fileSelectionButtonThemeExtension?.selectedStyle ??
                style)
            .stroke,
        shadow: (selectedStyle ??
                fileSelectionButtonThemeExtension?.selectedStyle ??
                style)
            .shadow,
      );
    }

    if (state.isInvalid) {
      // TODO(hpcl): enhance copyWith to copy only non-null attributes of an object
      style = style.copyWith(
        title: (invalidStyle ??
                fileSelectionButtonThemeExtension?.invalidStyle ??
                style)
            .title,
        subTitle: (invalidStyle ??
                fileSelectionButtonThemeExtension?.invalidStyle ??
                style)
            .subTitle,
        radius: (invalidStyle ??
                fileSelectionButtonThemeExtension?.invalidStyle ??
                style)
            .radius,
        padding: (invalidStyle ??
                fileSelectionButtonThemeExtension?.invalidStyle ??
                style)
            .padding,
        foregroundColors: (invalidStyle ??
                fileSelectionButtonThemeExtension?.invalidStyle ??
                style)
            .foregroundColors,
        backgroundColors: (invalidStyle ??
                fileSelectionButtonThemeExtension?.invalidStyle ??
                style)
            .backgroundColors,
        borderColors: (invalidStyle ??
                fileSelectionButtonThemeExtension?.invalidStyle ??
                style)
            .borderColors,
        stroke: (invalidStyle ??
                fileSelectionButtonThemeExtension?.invalidStyle ??
                style)
            .stroke,
        shadow: (invalidStyle ??
                fileSelectionButtonThemeExtension?.invalidStyle ??
                style)
            .shadow,
      );
    }

    return style;
  }

  Widget _border(
    BuildContext context,
    FileSelectionButtonStyle style,
    Widget child,
  ) {
    if (style.borderColors != null && style.stroke != null) {
      return DottedBorder(
        padding: EdgeInsets.zero,
        dashPattern: const [5, 5],
        strokeWidth: style.stroke!,
        color: style.borderColors!.color,
        borderType: BorderType.RRect,
        radius:
            style.radius?.resolve(TextDirection.ltr).bottomLeft ?? Radius.zero,
        strokeCap: StrokeCap.square,
        child: child,
      );
    } else {
      return child;
    }
  }

  @override
  Widget onBuild(BuildContext context, ButtonState state) {
    final style = negotiate(context, state);

    return Focus(
      onFocusChange: (hasFocus) =>
          hasFocus ? bloc(context).onFocus() : bloc(context).onUnfocus(),
      onKeyEvent: (focusNode, event) {
        if (event.logicalKey == LogicalKeyboardKey.enter && state.isFocused) {
          onPressed?.call(state.state);
          bloc(context).onClickUpOut();
          return KeyEventResult.handled;
        }
        return KeyEventResult.ignored;
      },
      child: MouseRegion(
        cursor: SystemMouseCursors.click,
        onEnter: (event) {
          bloc(context).onMouseEnter();
        },
        onExit: (event) {
          bloc(context).onMouseLeave();
        },
        child: GestureDetector(
          onTapDown: (details) {
            bloc(context).onClickDown();
          },
          onTapUp: (details) {
            onPressed?.call(state.state);
            bloc(context).onClickUpIn();
          },
          onTapCancel: () {
            onPressed?.call(state.state);
            bloc(context).onClickUpOut();
          },
          child: _border(
            context,
            style,
            DecoratedBox(
              decoration: BoxDecoration(
                color: style.backgroundColors?.color,
                // if no gradient colors => no default value
                gradient: (style.backgroundColors?.isGradient ?? false)
                    ? LinearGradient(
                        colors: style.backgroundColors!.colors,
                      )
                    : null,
                boxShadow: [
                  if (style.shadow != null) ...[style.shadow!]
                ],
                borderRadius: style.radius,
              ),
              child: ConstrainedBox(
                constraints: const BoxConstraints(
                  minWidth: 200,
                  minHeight: 50,
                ), // min sizes for Material buttons
                child: Padding(
                  padding: style.padding ?? EdgeInsets.zero,
                  child: Row(
                    mainAxisSize: mainAxisSize ?? MainAxisSize.min,
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    children: [
                      if (leading != null) ...[
                        leading ?? const SizedBox.shrink(),
                        Gap(style.padding?.horizontal ?? 10),
                      ],
                      Column(
                        mainAxisAlignment: MainAxisAlignment.spaceBetween,
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: [
                          /// Choose color
                          /// title.style.color ??
                          ///  buttonStyle.title.style.color ??
                          ///    context.textTheme.titleLarge.color
                          ///
                          /// Choose gradient
                          /// title.gradient ??
                          ///  buttonStyle.foregroundColor.colors ??
                          ///    null
                          ///
                          /// More infos in `negociate()` method
                          if (title != null) ...[
                            Gap(style.padding?.horizontal ?? 10),
                            Text(
                              title!.text,
                              style: title!.style ?? style.title,
                            ).toGradient(
                              LinearGradientHelper.fromNullableColors(
                                title?.gradient ??
                                    ((style.foregroundColors?.isGradient ??
                                            false)
                                        ? style.foregroundColors?.colors
                                        : null),
                              ),
                            ),
                          ],

                          /// Choose color
                          /// subTitle.style.color ??
                          ///  buttonStyle.subTitle.style.color ??
                          ///    context.textTheme.subTitleLarge.color
                          ///
                          /// Choose gradient
                          /// subTitle.gradient ??
                          ///  buttonStyle.foregroundColor.colors ??
                          ///    null
                          ///
                          /// More infos in `negociate()` method
                          if (subTitle != null) ...[
                            Gap(style.padding?.horizontal ?? 10),
                            Text(
                              subTitle!.text,
                              style: subTitle!.style ?? style.subTitle,
                            ).toGradient(
                              LinearGradientHelper.fromNullableColors(
                                subTitle?.gradient ??
                                    ((style.foregroundColors?.isGradient ??
                                            false)
                                        ? style.foregroundColors?.colors
                                        : null),
                              ),
                            ),
                          ],
                        ],
                      ),
                    ],
                  ),
                ),
              ),
            ),
          ),
        ),
      ),
    );
  }
}
